@@ -29,6 +29,7 @@
 #include "client.h"
 #include "common.h"
 
+#define DEFAULT_SOCKS5_PORT		1980
 #define FTP_RMT_CTL_PROXY_SUFFIX	"_ftp_remote_ctl_proxy"
 
 //client common config
@@ -58,4 +59,6 @@ struct proxy_service *get_proxy_service(const char *proxy_name);
 
 struct proxy_service *get_all_proxy_services();
 
+int validate_proxy(struct proxy_service *ps);
+
 #endif //_CONFIG_H_
