@@ -41,7 +41,6 @@
 #include "client.h"
 #include "uthash.h"
 #include "config.h"
-#include "const.h"
 #include "msg.h"
 #include "control.h"
 #include "uthash.h"
@@ -235,7 +234,7 @@ static void
 hb_sender_cb(evutil_socket_t fd, short event, void *arg)
 {
 	if (is_client_connected()) {
-		//debug(LOG_DEBUG, "ping frps");
+		debug(LOG_INFO, "ping frps");
 		ping(NULL);
 	}
 
