@@ -38,7 +38,7 @@ struct frp_coder {
 	uint8_t 	key[16];
 	char 		*salt;
 	uint8_t 	iv[16];
-	char 		*privilege_token;
+	char 		*token;
 };
 
 size_t get_encrypt_block_size();
@@ -47,7 +47,7 @@ int is_encoder_inited();
 int is_decoder_inited();
 struct frp_coder *init_main_encoder();
 struct frp_coder *init_main_decoder(const uint8_t *iv);
-struct frp_coder *new_coder(const char *privilege_token, const char *salt);
+struct frp_coder *new_coder(const char *token, const char *salt);
 uint8_t *encrypt_key(const char *token, size_t token_len, const char *salt, uint8_t *key, size_t key_len);
 uint8_t *encrypt_iv(uint8_t *iv_buf, size_t iv_len);
 size_t encrypt_data(const uint8_t *src_data, size_t srclen, struct frp_coder *encoder, uint8_t **ret);
