@@ -34,7 +34,6 @@
 #include <netinet/in.h>
 
 #include "msg.h"
-#include "const.h"
 #include "config.h"
 #include "debug.h"
 #include "common.h"
@@ -174,15 +173,37 @@ new_proxy_service_marshal(const struct proxy_service *np_req, char **msg)
 {
 	const char *tmp = NULL;
 	int  nret = 0;
+	char *path = NULL;
+	char *delimiter = ",";
+	char *save = NULL;
 	struct json_object *j_np_req = json_object_new_object();
 	if ( ! j_np_req)
 		return 0;
 	
 	JSON_MARSHAL_TYPE(j_np_req, "proxy_name", string, np_req->proxy_name);
-	JSON_MARSHAL_TYPE(j_np_req, "proxy_type", string, np_req->proxy_type);
+	// if proxy_type is socks5, set the proxy_type to tcp
+	if (strcmp(np_req->proxy_type, "socks5") == 0) {
+		JSON_MARSHAL_TYPE(j_np_req, "proxy_type", string, "tcp");
+	} else {
+		JSON_MARSHAL_TYPE(j_np_req, "proxy_type", string, np_req->proxy_type);
+	}
 	JSON_MARSHAL_TYPE(j_np_req, "use_encryption", boolean, np_req->use_encryption);
 	JSON_MARSHAL_TYPE(j_np_req, "use_compression", boolean, np_req->use_compression);
 
+	// if proxy_type is tcp, http, https and socks5, set group and group_key to j_np_req
+	if (strcmp(np_req->proxy_type, "tcp") == 0 || 
+		strcmp(np_req->proxy_type, "http") == 0 || 
+		strcmp(np_req->proxy_type, "https") == 0 || 
+		strcmp(np_req->proxy_type, "socks5") == 0) {
+		
+		if (np_req->group) {
+			JSON_MARSHAL_TYPE(j_np_req, "group", string, np_req->group);
+		}
+		if (np_req->group_key) {
+			JSON_MARSHAL_TYPE(j_np_req, "group_key", string, np_req->group_key);
+		}
+	}
+
 	if (is_ftp_proxy(np_req)) {
 		JSON_MARSHAL_TYPE(j_np_req, "remote_data_port", int, np_req->remote_data_port);
 	}
@@ -204,6 +225,11 @@ new_proxy_service_marshal(const struct proxy_service *np_req, char **msg)
 	json_object *j_location_array = json_object_new_array();
 	if (np_req->locations) {
 		json_object_object_add(j_np_req, "locations", j_location_array);
+		path = strtok_r(np_req->locations, delimiter, &save);
+		while (path) {
+			json_object_array_add(j_location_array, json_object_new_string(path));
+			path = strtok_r(NULL, delimiter, &save);
+		}
 	} else {
 		json_object_object_add(j_np_req, "locations", NULL);
 	}
