@@ -33,6 +33,7 @@
 #include "config.h"
 #include "debug.h"
 #include "control.h"
+#include "proxy.h"
 
 static uint8_t proto_version = 0;
 static uint8_t remote_go_away;
@@ -41,8 +42,6 @@ static uint32_t g_session_id = 1;
 static struct tmux_stream *cur_stream = NULL;
 static struct tmux_stream *all_stream;
 
-static uint32_t rx_ring_buffer_read(struct bufferevent *bev, struct ring_buffer *ring, uint32_t len);
-static uint32_t tx_ring_buffer_write(struct bufferevent *bev, struct ring_buffer *ring, uint32_t len);
 
 void
 add_stream(struct tmux_stream *stream)
@@ -308,12 +307,12 @@ send_window_update(struct bufferevent *bout, struct tmux_stream *stream, uint32_
 	//				flags, stream->id, delta, stream->recv_window, length);
 }
 
-static int
+int
 rx_ring_buffer_pop(struct ring_buffer *ring, uint8_t *data, uint32_t len)
 {
 	assert(ring->sz >= len);
-	assert(data != NULL);
-	
+	assert(data);
+
 	uint32_t i = 0;
 	while(i < len) {
 		data[i] = ring->data[ring->cur++];
@@ -331,8 +330,16 @@ static int
 process_data(struct tmux_stream *stream, uint32_t length, uint16_t flags, 
 				void (*fn)(uint8_t *, int, void *), void *param)
 {
+    if(!stream){
+        return 0;
+    }
+	uint32_t id = stream->id;
+
 	if (!process_flags(flags, stream)) return 0;
 
+	if(!get_stream_by_id(id)){
+		return length;
+	}
 
 	if (length > stream->recv_window) {
 		debug(LOG_ERR, "receive window exceed (remain %d, recv %d)", stream->recv_window, length);
@@ -341,18 +348,27 @@ process_data(struct tmux_stream *stream, uint32_t length, uint16_t flags,
 	
 	stream->recv_window -= length;
 
+	uint32_t nret = 0;
 	struct proxy_client *pc = (struct proxy_client *)param;
-	if (!pc || (pc && !pc->local_proxy_bev)) {
+	if (!pc || (pc && !pc->local_proxy_bev && !is_socks5_proxy(pc->ps))) {
 		uint8_t *data = (uint8_t *)calloc(length, 1);
-		rx_ring_buffer_pop(&stream->rx_ring, data, length);
+		nret = rx_ring_buffer_pop(&stream->rx_ring, data, length);
 		fn(data, length, pc);
 		free(data);
+	} else if (is_socks5_proxy(pc->ps)) {
+		// if pc's type is socks5, we should send data to socks5 client
+		nret = handle_ss5(pc, &stream->rx_ring, length);
 	} else {
-		tx_ring_buffer_write(pc->local_proxy_bev, &stream->rx_ring, length);
+		nret = tx_ring_buffer_write(pc->local_proxy_bev, &stream->rx_ring, length);
+
 	}
-	
+
+	if (nret != length) {
+		debug(LOG_INFO, "send data to local proxy not equal, nret %d, length %d", nret, length);
+	}
+
 	struct bufferevent *bout = get_main_control()->connect_bev;
-	send_window_update(bout, stream, length);	
+	send_window_update(bout, stream, nret);	
 
 	return length;
 }
@@ -363,9 +379,15 @@ incr_send_window(struct bufferevent *bev, struct tcp_mux_header *tmux_hdr, uint1
     if(!stream){
         return 0;
     }
+	uint32_t id = stream->id;
 
 	if (!process_flags(flags, stream))
 		return 0;
+
+	if(!get_stream_by_id(id)){
+		return 1;
+	}
+
 	uint32_t length = ntohl(tmux_hdr->length);
 
 	if (stream->send_window == 0) bufferevent_enable(bev, EV_READ);
@@ -495,7 +517,7 @@ tx_ring_buffer_append(struct ring_buffer *ring, uint8_t *data, uint32_t len)
 	return i;
 }
 
-static uint32_t
+uint32_t
 rx_ring_buffer_read(struct bufferevent *bev, struct ring_buffer *ring, uint32_t len)
 {
 	if (ring->sz == RBUF_SIZE) {
@@ -522,7 +544,7 @@ rx_ring_buffer_read(struct bufferevent *bev, struct ring_buffer *ring, uint32_t
 
 }
 
-static uint32_t
+uint32_t
 tx_ring_buffer_write(struct bufferevent *bev, struct ring_buffer *ring, uint32_t len)
 {
 	if (ring->sz == 0) {
@@ -535,6 +557,7 @@ tx_ring_buffer_write(struct bufferevent *bev, struct ring_buffer *ring, uint32_t
 		len = ring->sz;
 	}
 
+	uint32_t nwrite = len;
 	while(len > 0) {
 		bufferevent_write(bev, &ring->data[ring->cur++], 1);
 		len--;
@@ -546,7 +569,7 @@ tx_ring_buffer_write(struct bufferevent *bev, struct ring_buffer *ring, uint32_t
 		}
 	}
 
-	return len;
+	return nwrite - len;
 }
 
 uint32_t 
